"""
论文标准MLP模型 - TPAMI 2025

从 methods/models/paper_mlp.py 迁移到 src/

符合论文 "Fundamentals and Experimental Analysis of Federated Learning Algorithms:
A Comparative Study on Non-IID Data Silos" (TPAMI 2025) Table III实验设置
"""
import torch
import torch.nn as nn
from ...registry import model


@model(
    name='paper_mlp',
    description='论文标准MLP模型(TPAMI 2025) - 适配所有表格数据集',
    version='1.0',
    task='classification'
)
class PaperMLP(nn.Module):
    """
    论文标准MLP模型 - 适配所有表格数据集

    Architecture (TPAMI 2025 Paper, Section VI):
    -------------------------------------------
    "For the tabular datasets, we employ a standard MLP with
    three hidden layers (32, 16, and 8 units)."

    - FC1: input_dim → 32, ReLU
    - FC2: 32 → 16, ReLU
    - FC3: 16 → 8, ReLU
    - FC4: 8 → num_classes

    支持的数据集:
    - Adult: 99特征, 2分类
    - FCUBE: 3特征, 2分类
    """

    def __init__(self,
                 input_dim: int,
                 num_classes: int = 2):
        """
        初始化论文标准MLP模型

        Args:
            input_dim: 输入特征维度
            num_classes: 分类数量
        """
        super().__init__()

        # 全连接层(论文标准: 32 → 16 → 8)
        self.fc1 = nn.Linear(input_dim, 32)
        self.fc2 = nn.Linear(32, 16)
        self.fc3 = nn.Linear(16, 8)
        self.fc4 = nn.Linear(8, num_classes)

    def forward(self, x):
        """
        前向传播

        Args:
            x: 输入张量, shape=(batch_size, input_dim)

        Returns:
            logits: 分类logits, shape=(batch_size, num_classes)
        """
        # 确保输入是2D张量
        if x.dim() == 1:
            x = x.unsqueeze(0)

        # 第一层
        x = self.fc1(x)
        x = torch.relu(x)

        # 第二层
        x = self.fc2(x)
        x = torch.relu(x)

        # 第三层
        x = self.fc3(x)
        x = torch.relu(x)

        # 输出层
        x = self.fc4(x)

        return x


# 为不同数据集创建便捷的工厂类

@model(
    name='adult_paper_mlp',
    description='Adult数据集论文标准MLP(99特征,2分类)',
    version='1.0',
    task='classification',
    input_shape=(99,),
    output_shape=(2,)
)
class Adult_PaperMLP(PaperMLP):
    """Adult数据集的论文标准MLP"""
    def __init__(self, num_classes: int = 2):
        super().__init__(
            input_dim=99,
            num_classes=num_classes
        )


@model(
    name='fcube_paper_mlp',
    description='FCUBE数据集论文标准MLP(3特征,2分类)',
    version='1.0',
    task='classification',
    input_shape=(3,),
    output_shape=(2,)
)
class FCUBE_PaperMLP(PaperMLP):
    """FCUBE数据集的论文标准MLP"""
    def __init__(self, num_classes: int = 2):
        super().__init__(
            input_dim=3,
            num_classes=num_classes
        )



"""
论文标准CNN模型 - TPAMI 2025

从 methods/models/paper_cnn.py 迁移到 src/

符合论文 "Fundamentals and Experimental Analysis of Federated Learning Algorithms:
A Comparative Study on Non-IID Data Silos" (TPAMI 2025) Table III实验设置
"""
import torch
import torch.nn as nn
from ...registry import model


@model(
    name='paper_cnn',
    description='论文标准CNN模型(TPAMI 2025) - 适配所有图像数据集',
    version='1.0',
    task='classification'
)
class PaperCNN(nn.Module):
    """
    论文标准CNN模型 - 适配所有图像数据集

    Architecture (TPAMI 2025 Paper, Section VI):
    -------------------------------------------
    - Conv1: in_channels → 6, kernel_size=5
    - ReLU + MaxPool(2×2)
    - Conv2: 6 → 16, kernel_size=5
    - ReLU + MaxPool(2×2)
    - Flatten
    - FC1: flatten_size → 120
    - ReLU
    - FC2: 120 → 84
    - ReLU
    - FC3: 84 → num_classes

    支持的数据集:
    - MNIST: 1×28×28 → flatten_size=256
    - FMNIST: 1×28×28 → flatten_size=256
    - CIFAR-10: 3×32×32 → flatten_size=400
    - SVHN: 3×32×32 → flatten_size=400
    - CINIC-10: 3×32×32 → flatten_size=400
    """

    def __init__(self,
                 num_classes: int = 10,
                 in_channels: int = 3,
                 input_height: int = 32,
                 input_width: int = 32):
        """
        初始化论文标准CNN模型

        Args:
            num_classes: 分类数量
            in_channels: 输入通道数(1为灰度图,3为彩色图)
            input_height: 输入图像高度
            input_width: 输入图像宽度
        """
        super().__init__()

        # 卷积层1: in_channels → 6, 5×5卷积核
        self.conv1 = nn.Conv2d(in_channels, 6, kernel_size=5)
        self.pool1 = nn.MaxPool2d(2, 2)

        # 卷积层2: 6 → 16, 5×5卷积核
        self.conv2 = nn.Conv2d(6, 16, kernel_size=5)
        self.pool2 = nn.MaxPool2d(2, 2)

        # 计算卷积后的特征图尺寸
        conv_output_h = ((input_height - 4) // 2 - 4) // 2
        conv_output_w = ((input_width - 4) // 2 - 4) // 2
        flatten_size = 16 * conv_output_h * conv_output_w

        # 全连接层(论文标准: 120 → 84 → num_classes)
        self.fc1 = nn.Linear(flatten_size, 120)
        self.fc2 = nn.Linear(120, 84)
        self.fc3 = nn.Linear(84, num_classes)

    def forward(self, x):
        """前向传播"""
        # 第一组卷积+池化
        x = self.conv1(x)
        x = torch.relu(x)
        x = self.pool1(x)

        # 第二组卷积+池化
        x = self.conv2(x)
        x = torch.relu(x)
        x = self.pool2(x)

        # 展平
        x = x.view(x.size(0), -1)

        # 全连接层
        x = self.fc1(x)
        x = torch.relu(x)
        x = self.fc2(x)
        x = torch.relu(x)
        x = self.fc3(x)

        return x


# 为不同数据集创建便捷的工厂类

@model(
    name='cifar10_paper_cnn',
    description='CIFAR-10论文标准CNN(3×32×32输入,10分类)',
    version='1.0',
    task='classification',
    input_shape=(3, 32, 32),
    output_shape=(10,)
)
class CIFAR10_PaperCNN(PaperCNN):
    """CIFAR-10数据集的论文标准CNN"""
    def __init__(self, num_classes: int = 10):
        super().__init__(
            num_classes=num_classes,
            in_channels=3,
            input_height=32,
            input_width=32
        )


@model(
    name='mnist_paper_cnn',
    description='MNIST论文标准CNN(1×28×28输入,10分类)',
    version='1.0',
    task='classification',
    input_shape=(1, 28, 28),
    output_shape=(10,)
)
class MNIST_PaperCNN(PaperCNN):
    """MNIST数据集的论文标准CNN"""
    def __init__(self, num_classes: int = 10):
        super().__init__(
            num_classes=num_classes,
            in_channels=1,
            input_height=28,
            input_width=28
        )



"""ResNet models for CIFAR datasets"""
import torch
import torch.nn as nn
import torch.nn.functional as F

from ...registry.decorators import model


class BasicBlock(nn.Module):
    """Basic Block for ResNet18/34"""
    expansion = 1

    def __init__(self, in_planes, planes, stride=1):
        super(BasicBlock, self).__init__()
        self.conv1 = nn.Conv2d(in_planes, planes, kernel_size=3, stride=stride, padding=1, bias=False)
        self.bn1 = nn.BatchNorm2d(planes)
        self.conv2 = nn.Conv2d(planes, planes, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn2 = nn.BatchNorm2d(planes)

        self.shortcut = nn.Sequential()
        if stride != 1 or in_planes != self.expansion * planes:
            self.shortcut = nn.Sequential(
                nn.Conv2d(in_planes, self.expansion * planes, kernel_size=1, stride=stride, bias=False),
                nn.BatchNorm2d(self.expansion * planes)
            )

    def forward(self, x):
        out = F.relu(self.bn1(self.conv1(x)))
        out = self.bn2(self.conv2(out))
        out += self.shortcut(x)
        out = F.relu(out)
        return out


class ResNet(nn.Module):
    """ResNet architecture for CIFAR (32x32 images)"""

    def __init__(self, block, num_blocks, num_classes=10):
        super(ResNet, self).__init__()
        self.in_planes = 64

        # Initial conv layer (for CIFAR: kernel=3, stride=1, no maxpool)
        self.conv1 = nn.Conv2d(3, 64, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn1 = nn.BatchNorm2d(64)

        # ResNet layers
        self.layer1 = self._make_layer(block, 64, num_blocks[0], stride=1)
        self.layer2 = self._make_layer(block, 128, num_blocks[1], stride=2)
        self.layer3 = self._make_layer(block, 256, num_blocks[2], stride=2)
        self.layer4 = self._make_layer(block, 512, num_blocks[3], stride=2)

        # Final classifier
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.fc = nn.Linear(512 * block.expansion, num_classes)

    def _make_layer(self, block, planes, num_blocks, stride):
        strides = [stride] + [1] * (num_blocks - 1)
        layers = []
        for stride in strides:
            layers.append(block(self.in_planes, planes, stride))
            self.in_planes = planes * block.expansion
        return nn.Sequential(*layers)

    def forward(self, x):
        out = F.relu(self.bn1(self.conv1(x)))
        out = self.layer1(out)
        out = self.layer2(out)
        out = self.layer3(out)
        out = self.layer4(out)
        out = self.avgpool(out)
        out = torch.flatten(out, 1)
        out = self.fc(out)
        return out


@model(
    name='resnet18',
    description='ResNet18 for CIFAR-10/100 (32x32 images)',
    task='classification',
    version='1.0'
)
class ResNet18_CIFAR(ResNet):
    """ResNet18 for CIFAR datasets

    Adapted for 32x32 images (CIFAR-10/100):
    - Uses kernel=3, stride=1 in conv1 (no stride=2)
    - No initial maxpool layer
    - 4 residual layers: [2, 2, 2, 2] blocks

    Args:
        num_classes: Number of output classes (10 for CIFAR-10, 100 for CIFAR-100)
    """

    def __init__(self, num_classes=10):
        super(ResNet18_CIFAR, self).__init__(
            block=BasicBlock,
            num_blocks=[2, 2, 2, 2],
            num_classes=num_classes
        )


@model(
    name='resnet34',
    description='ResNet34 for CIFAR-10/100 (32x32 images)',
    task='classification',
    version='1.0'
)
class ResNet34_CIFAR(ResNet):
    """ResNet34 for CIFAR datasets

    Args:
        num_classes: Number of output classes
    """

    def __init__(self, num_classes=10):
        super(ResNet34_CIFAR, self).__init__(
            block=BasicBlock,
            num_blocks=[3, 4, 6, 3],
            num_classes=num_classes
        )



"""
CIFAR-10 CNN 分类模型

支持 VFL/SplitNN 的模型分割（通过 features/classifier 结构）
"""
import torch
import torch.nn as nn
import torch.nn.functional as F

from ...registry.decorators import model


@model(
    name='CNN',
    description='ResNet风格的CNN模型',
    task='classification',
    version='1.0'
)
class CNNModel(nn.Module):
    """ResNet风格的CNN模型

    基于ResNet思想的轻量级CNN，适合联邦学习
    输入: (batch, 3, 32, 32) - 适用于CIFAR-10, SVHN等
    输出: (batch, num_classes)
    
    支持 VFL/SplitNN 分割：通过 features/classifier 属性
    """

    def __init__(self, num_classes: int = 10):
        super(CNNModel, self).__init__()

        # 卷积部分（用于 VFL 分割）
        self.features = nn.Sequential(
            # 第一个卷积块
            nn.Conv2d(3, 64, kernel_size=3, padding=1),
            nn.BatchNorm2d(64),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 32x32 -> 16x16
            
            # 第二个卷积块
            nn.Conv2d(64, 128, kernel_size=3, padding=1),
            nn.BatchNorm2d(128),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 16x16 -> 8x8
            
            # 第三个卷积块
            nn.Conv2d(128, 256, kernel_size=3, padding=1),
            nn.BatchNorm2d(256),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 8x8 -> 4x4
            
            # 第四个卷积块
            nn.Conv2d(256, 512, kernel_size=3, padding=1),
            nn.BatchNorm2d(512),
            nn.ReLU(inplace=True),
            
            # 全局平均池化
            nn.AdaptiveAvgPool2d((1, 1)),  # 4x4 -> 1x1
        )
        
        # 全连接部分（用于 VFL 分割）
        self.classifier = nn.Sequential(
            nn.Dropout(0.5),
            nn.Linear(512, num_classes),
        )

    def forward(self, x):
        x = self.features(x)
        x = torch.flatten(x, 1)  # (batch, 512)
        x = self.classifier(x)
        return x


@model(
    name='SimpleCNN',
    description='简单的3层CNN模型',
    task='classification',
    version='1.0'
)
class SimpleCNNModel(nn.Module):
    """简单的3层CNN模型

    更轻量的版本，适合快速实验
    输入: (batch, 3, 32, 32) - 适用于CIFAR-10, SVHN等
    输出: (batch, num_classes)
    
    支持 VFL/SplitNN 分割：通过 features/classifier 属性
    """

    def __init__(self, num_classes: int = 10):
        super(SimpleCNNModel, self).__init__()

        # 卷积部分
        self.features = nn.Sequential(
            nn.Conv2d(3, 32, kernel_size=3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 32->16
            
            nn.Conv2d(32, 64, kernel_size=3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 16->8
            
            nn.Conv2d(64, 64, kernel_size=3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),  # 8->4
        )
        
        # 全连接部分
        self.classifier = nn.Sequential(
            nn.Linear(64 * 4 * 4, 128),
            nn.ReLU(inplace=True),
            nn.Dropout(0.25),
            nn.Linear(128, num_classes),
        )

    def forward(self, x):
        x = self.features(x)
        x = torch.flatten(x, 1)
        x = self.classifier(x)
        return x


"""
配置系统 v3.0

重构设计：
- types.py: 纯数据类定义
- manager.py: 配置管理器（加载、合并、同步、验证）

主要特性：
1. 层次化配置（GlobalConfig → NodeConfig）
2. 自动同步共享字段（exp_name, run_name, log_dir）
3. 配置继承（extend 字段）
4. 完整的类型提示和验证

Quick Start:
    # 从文件加载
    from config import load_config
    config = load_config("configs/trainer.yaml")
    
    # 从字典创建
    from config import load_config_from_dict
    config = load_config_from_dict({
        "node_id": "trainer",
        "global": {"exp_name": "my_experiment"},
        "listen": {"port": 50051},
        "trainer": {"type": "federated.trainer.fedavg"},
        "aggregator": {"type": "federated.aggregator.fedavg"},
    })
    
    # 访问自动同步的字段
    print(config.exp_name)              # "my_experiment"
    print(config.logging.exp_name)      # "my_experiment" (自动同步)
    
    # 使用管理器
    from config import ConfigManager
    manager = ConfigManager()
    config = manager.load("config.yaml")
    manager.save(config, "output.yaml")
"""

# ==================== 类型定义 ====================

from .types import (
    # 枚举
    LogLevel,
    TransportMode,
    BackoffStrategy,
    NodeRole,
    
    # 全局配置
    GlobalConfig,
    
    # 日志配置
    LogConfig,
    
    # 追踪配置
    TrackerBackendConfig,
    MLflowConfig,
    WandbConfig,
    TensorBoardConfig,
    TrackerConfig,
    
    # 传输配置
    GrpcConfig,
    TransportConfig,
    ConnectionRetryConfig,
    
    # 组件配置
    ComponentConfig,
    DatasetConfig,
    CallbackConfig,
    
    # 通信节点配置
    NodeCommConfig,
    
    # 节点配置
    NodeConfig,
)

# ==================== 管理器 ====================

from .manager import (
    # 异常
    ConfigError,
    ConfigValidationError,
    ConfigLoadError,
    
    # 管理器
    ConfigManager,
    get_default_manager,
    
    # 便捷函数
    load_config,
    load_config_from_dict,
    save_config,
    validate_config,
    config_to_dict,
    
    # 向后兼容
    load_node_config,
    deep_merge,
    create_client_config,
)


# ==================== 向后兼容别名 ====================

# 保留旧的类型别名
FederationConfig = NodeConfig
LoggingConfig = LogConfig


# ==================== 版本信息 ====================

__version__ = "3.0.0"


# ==================== 导出 ====================

__all__ = [
    # 版本
    "__version__",
    
    # 枚举
    "LogLevel",
    "TransportMode",
    "BackoffStrategy",
    "NodeRole",
    
    # 配置类
    "GlobalConfig",
    "LogConfig",
    "TrackerBackendConfig",
    "MLflowConfig",
    "WandbConfig",
    "TensorBoardConfig",
    "TrackerConfig",
    "GrpcConfig",
    "TransportConfig",
    "ConnectionRetryConfig",
    "ComponentConfig",
    "DatasetConfig",
    "CallbackConfig",
    "NodeCommConfig",
    "NodeConfig",
    
    # 异常
    "ConfigError",
    "ConfigValidationError",
    "ConfigLoadError",
    
    # 管理器
    "ConfigManager",
    "get_default_manager",
    
    # 便捷函数
    "load_config",
    "load_config_from_dict",
    "save_config",
    "validate_config",
    "config_to_dict",
    
    # 向后兼容
    "load_node_config",
    "deep_merge",
    "create_client_config",
    "FederationConfig",
    "LoggingConfig",
]



"""
Pickle 序列化器
"""

import pickle
import sys
from typing import Any

from .base import Serializer


class PickleSerializer(Serializer):
    """Pickle 序列化器"""

    def __init__(self):
        super().__init__()
        # 添加模块别名：federation -> src
        # 这样即使序列化时使用 federation.xxx，反序列化时也能找到 src.xxx
        if 'federation' not in sys.modules and 'src' in sys.modules:
            sys.modules['federation'] = sys.modules['src']

    @property
    def name(self) -> str:
        return "pickle"

    def serialize(self, obj: Any) -> bytes:
        return pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)

    def deserialize(self, data: bytes) -> Any:
        if not data:
            return None
        return pickle.loads(data)



"""
MNIST CNN 分类模型

支持 VFL/SplitNN 的模型分割（通过 features/classifier 结构）
"""
import torch
import torch.nn as nn
from ...registry import model


@model(
    name='mnist_cnn',
    description='MNIST CNN分类模型',
    version='1.0',
    task='classification',
    input_shape=(1, 28, 28),
    output_shape=(10,)
)
class MNISTCNNModel(nn.Module):
    """MNIST CNN模型

    一个简单的卷积神经网络,用于MNIST手写数字分类。

    网络结构:
    - features: Conv2d(1, 32) + ReLU + Conv2d(32, 64) + ReLU + MaxPool + Dropout
    - classifier: Linear(64*14*14, 128) + ReLU + Dropout + Linear(128, 10)
    
    支持 VFL/SplitNN 分割：通过 features/classifier 属性
    """

    def __init__(self, num_classes: int = 10):
        super().__init__()

        # 卷积部分（用于 VFL 分割）
        self.features = nn.Sequential(
            nn.Conv2d(1, 32, kernel_size=3, padding=1),
            nn.ReLU(inplace=True),
            nn.Conv2d(32, 64, kernel_size=3, padding=1),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2, 2),
            nn.Dropout(0.25),
        )
        
        # 全连接部分（用于 VFL 分割）
        self.classifier = nn.Sequential(
            nn.Linear(64 * 14 * 14, 128),
            nn.ReLU(inplace=True),
            nn.Dropout(0.5),
            nn.Linear(128, num_classes),
        )

    def forward(self, x):
        """前向传播"""
        x = self.features(x)
        x = x.view(x.size(0), -1)  # flatten: (batch, 64*14*14)
        x = self.classifier(x)
        return x


"""
Node 通信层配置定义
"""

from dataclasses import dataclass, field
from typing import Optional, Dict, List, Any
import yaml


@dataclass
class TlsConfig:
    """TLS 配置"""
    
    enabled: bool = False
    cert_file: Optional[str] = None
    key_file: Optional[str] = None
    ca_file: Optional[str] = None
    mutual_tls: bool = False


@dataclass
class MemoryTransportConfig:
    """Memory 传输配置"""
    
    zero_copy: bool = True                       # 零拷贝模式
    simulate_latency: bool = False               # 模拟延迟
    latency_ms: float = 0.0                      # 延迟毫秒数


@dataclass
class GrpcTransportConfig:
    """gRPC 传输配置"""

    host: str = "0.0.0.0"
    port: int = 50051
    max_workers: int = 10
    max_message_length: int = 104857600  # 100MB
    tls: TlsConfig = field(default_factory=TlsConfig)

    # 双线程配置
    dual_thread_enabled: bool = True  # 默认启用双线程优化

    # 心跳配置（gRPC 专属）
    heartbeat_enabled: bool = True
    heartbeat_interval: float = 5.0       # 心跳间隔（秒）
    heartbeat_timeout: float = 30.0       # 超时时间（秒）
    heartbeat_check_interval: float = 10.0  # 健康检查间隔（秒）

    # 连接失败自动退出配置
    max_connection_wait_time: float = 300.0  # 最大连接等待时间（秒），默认5分钟
    auto_shutdown_on_failure: bool = True    # 连接失败后是否自动shutdown
    critical_peers: List[str] = field(default_factory=list)  # 关键节点列表（如trainer）


@dataclass
class TransportConfig:
    """传输层配置"""
    
    mode: str = "memory"                         # memory | grpc
    memory: MemoryTransportConfig = field(default_factory=MemoryTransportConfig)
    grpc: GrpcTransportConfig = field(default_factory=GrpcTransportConfig)


@dataclass
class MethodSerializationConfig:
    """方法序列化配置"""
    
    serializer: str = "json"
    compress: bool = False


@dataclass
class SerializationConfig:
    """序列化配置"""

    default: str = "pickle"                      # 默认序列化器（联邦学习使用pickle处理复杂对象）
    methods: Dict[str, MethodSerializationConfig] = field(default_factory=dict)


@dataclass
class RetryConfig:
    """重试配置"""
    
    enabled: bool = False
    max_retries: int = 3
    backoff: float = 1.0


@dataclass
class AuthConfig:
    """认证配置"""
    
    mode: str = "token"                          # token | mutual_tls | custom
    token: Optional[str] = None                  # 静态 token
    # token_provider: Callable[[], str] 可通过代码设置


@dataclass
class InterceptorConfig:
    """拦截器配置"""
    
    logging: bool = True
    metrics: bool = False
    auth: bool = False
    auth_config: AuthConfig = field(default_factory=AuthConfig)
    retry: RetryConfig = field(default_factory=RetryConfig)
    custom: List[str] = field(default_factory=list)


@dataclass
class HeartbeatConfig:
    """心跳配置"""
    
    enabled: bool = False
    interval: float = 30.0                       # 心跳间隔（秒）
    timeout: float = 90.0                        # 超时时间（秒）


@dataclass
class MethodOptions:
    """方法选项"""
    
    serializer: Optional[str] = None             # 指定序列化器
    timeout: Optional[float] = None              # 超时时间
    require_auth: bool = False                   # 是否需要认证


@dataclass
class NodeConfig:
    """节点配置"""

    # 基本配置
    node_id: str = ""                            # 节点 ID
    debug: bool = False                          # 调试模式

    # 超时配置
    default_timeout: float = 300.0                # 默认调用超时（秒）

    # 网络地址
    advertised_address: Optional[str] = None     # 对外公布的地址（NAT/容器环境）
    listen: Optional[Dict[str, Any]] = None      # 监听配置 {"host": "0.0.0.0", "port": 50051}

    # 传输层配置
    transport: TransportConfig = field(default_factory=TransportConfig)

    # 序列化配置
    serialization: SerializationConfig = field(default_factory=SerializationConfig)

    # 拦截器配置
    interceptors: InterceptorConfig = field(default_factory=InterceptorConfig)

    # 心跳配置
    heartbeat: HeartbeatConfig = field(default_factory=HeartbeatConfig)


def _parse_tls_config(data: Dict[str, Any]) -> TlsConfig:
    """解析 TLS 配置"""
    if not data:
        return TlsConfig()
    return TlsConfig(
        enabled=data.get("enabled", False),
        cert_file=data.get("cert_file"),
        key_file=data.get("key_file"),
        ca_file=data.get("ca_file"),
        mutual_tls=data.get("mutual_tls", False),
    )


def _parse_memory_transport_config(data: Dict[str, Any]) -> MemoryTransportConfig:
    """解析 Memory 传输配置"""
    if not data:
        return MemoryTransportConfig()
    return MemoryTransportConfig(
        zero_copy=data.get("zero_copy", True),
        simulate_latency=data.get("simulate_latency", False),
        latency_ms=data.get("latency_ms", 0.0),
    )


def _parse_grpc_transport_config(data: Dict[str, Any]) -> GrpcTransportConfig:
    """解析 gRPC 传输配置"""
    if not data:
        return GrpcTransportConfig()

    # 解析双线程配置
    dual_thread = data.get("dual_thread", {})
    dual_thread_enabled = dual_thread.get("enabled", True)  # 默认启用

    # 解析心跳配置
    heartbeat = data.get("heartbeat", {})

    return GrpcTransportConfig(
        host=data.get("host", "0.0.0.0"),
        port=data.get("port", 50051),
        max_workers=data.get("max_workers", 10),
        max_message_length=data.get("max_message_length", 104857600),
        tls=_parse_tls_config(data.get("tls", {})),

        # 双线程配置
        dual_thread_enabled=dual_thread_enabled,

        # 心跳配置
        heartbeat_enabled=heartbeat.get("enabled", True),
        heartbeat_interval=heartbeat.get("interval", 5.0),
        heartbeat_timeout=heartbeat.get("timeout", 30.0),
        heartbeat_check_interval=heartbeat.get("check_interval", 10.0),

        # 连接失败自动退出配置
        max_connection_wait_time=heartbeat.get("max_connection_wait_time", 300.0),
        auto_shutdown_on_failure=heartbeat.get("auto_shutdown_on_failure", True),
        critical_peers=heartbeat.get("critical_peers", []),
    )


def _parse_transport_config(data: Dict[str, Any]) -> TransportConfig:
    """解析传输层配置"""
    if not data:
        return TransportConfig()
    return TransportConfig(
        mode=data.get("mode", "memory"),
        memory=_parse_memory_transport_config(data.get("memory", {})),
        grpc=_parse_grpc_transport_config(data.get("grpc", {})),
    )


def _parse_method_serialization_config(data: Dict[str, Any]) -> MethodSerializationConfig:
    """解析方法序列化配置"""
    return MethodSerializationConfig(
        serializer=data.get("serializer", "json"),
        compress=data.get("compress", False),
    )


def _parse_serialization_config(data: Dict[str, Any]) -> SerializationConfig:
    """解析序列化配置"""
    if not data:
        return SerializationConfig()

    methods = {}
    for method_name, method_data in data.get("methods", {}).items():
        methods[method_name] = _parse_method_serialization_config(method_data)

    return SerializationConfig(
        default=data.get("default", "pickle"),  # 默认使用pickle而不是json
        methods=methods,
    )


def _parse_retry_config(data: Dict[str, Any]) -> RetryConfig:
    """解析重试配置"""
    if not data:
        return RetryConfig()
    return RetryConfig(
        enabled=data.get("enabled", False),
        max_retries=data.get("max_retries", 3),
        backoff=data.get("backoff", 1.0),
    )


def _parse_auth_config(data: Dict[str, Any]) -> AuthConfig:
    """解析认证配置"""
    if not data:
        return AuthConfig()
    return AuthConfig(
        mode=data.get("mode", "token"),
        token=data.get("token"),
    )


def _parse_interceptor_config(data: Dict[str, Any]) -> InterceptorConfig:
    """解析拦截器配置"""
    if not data:
        return InterceptorConfig()
    return InterceptorConfig(
        logging=data.get("logging", True),
        metrics=data.get("metrics", False),
        auth=data.get("auth", False),
        auth_config=_parse_auth_config(data.get("auth_config", {})),
        retry=_parse_retry_config(data.get("retry", {})),
        custom=data.get("custom", []),
    )


def _parse_heartbeat_config(data: Dict[str, Any]) -> HeartbeatConfig:
    """解析心跳配置"""
    if not data:
        return HeartbeatConfig()
    return HeartbeatConfig(
        enabled=data.get("enabled", False),
        interval=data.get("interval", 30.0),
        timeout=data.get("timeout", 90.0),
    )


def load_config(path: str) -> NodeConfig:
    """从 YAML 文件加载配置"""
    with open(path, "r") as f:
        data = yaml.safe_load(f)
    
    return NodeConfig(
        node_id=data.get("node_id", ""),
        debug=data.get("debug", False),
        default_timeout=data.get("default_timeout", 300.0),
        advertised_address=data.get("advertised_address"),
        transport=_parse_transport_config(data.get("transport", {})),
        serialization=_parse_serialization_config(data.get("serialization", {})),
        interceptors=_parse_interceptor_config(data.get("interceptors", {})),
        heartbeat=_parse_heartbeat_config(data.get("heartbeat", {})),
    )



"""
MNIST LeNet模型 - 符合TPAMI 2025论文标准

从 methods/models/mnist_lenet.py 迁移到 src/

基于论文 "Fundamentals and Experimental Analysis of Federated Learning Algorithms:
A Comparative Study on Non-IID Data Silos" (TPAMI 2025) 的TABLE III实验设置
"""
import torch
import torch.nn as nn
from ...registry import model


@model(
    name='mnist_lenet',
    description='MNIST LeNet模型(TPAMI 2025论文标准架构)',
    version='1.0',
    task='classification',
    input_shape=(1, 28, 28),
    output_shape=(10,)
)
class MNISTLeNetModel(nn.Module):
    """
    MNIST LeNet模型 - 符合论文TABLE III使用的标准架构

    Architecture (from TPAMI 2025 paper, Section VI):
    -----------------------------------------------
    - 两个 5×5 卷积层(6和16通道)
    - 两个全连接层(120和84单元)
    - 最后一层为分类层

    Detailed Structure:
    ------------------
    Conv1: 1 → 6 channels, kernel_size=5
    ReLU + MaxPool(2×2)
    Conv2: 6 → 16 channels, kernel_size=5
    ReLU + MaxPool(2×2)
    Flatten: 16×4×4 = 256
    FC1: 256 → 120, ReLU
    FC2: 120 → 84, ReLU
    FC3: 84 → 10 (output layer)

    Parameters: ~61K
    """

    def __init__(self, num_classes: int = 10):
        super().__init__()

        # LeNet-like架构(严格按照论文标准)
        # 卷积层1: 1→6 通道, 5×5卷积核
        self.conv1 = nn.Conv2d(1, 6, kernel_size=5)     # 28×28 → 24×24
        self.pool1 = nn.MaxPool2d(2, 2)                 # 24×24 → 12×12

        # 卷积层2: 6→16 通道, 5×5卷积核
        self.conv2 = nn.Conv2d(6, 16, kernel_size=5)    # 12×12 → 8×8
        self.pool2 = nn.MaxPool2d(2, 2)                 # 8×8 → 4×4

        # 全连接层 (论文标准: 120 → 84 → 10)
        self.fc1 = nn.Linear(16 * 4 * 4, 120)           # 256 → 120
        self.fc2 = nn.Linear(120, 84)                   # 120 → 84
        self.fc3 = nn.Linear(84, num_classes)           # 84 → 10

    def forward(self, x):
        """
        前向传播

        Args:
            x: 输入张量, shape=(batch_size, 1, 28, 28)

        Returns:
            logits: 分类logits, shape=(batch_size, num_classes)
        """
        # 第一组卷积+池化
        x = self.conv1(x)                               # (B, 1, 28, 28) → (B, 6, 24, 24)
        x = torch.relu(x)
        x = self.pool1(x)                               # (B, 6, 24, 24) → (B, 6, 12, 12)

        # 第二组卷积+池化
        x = self.conv2(x)                               # (B, 6, 12, 12) → (B, 16, 8, 8)
        x = torch.relu(x)
        x = self.pool2(x)                               # (B, 16, 8, 8) → (B, 16, 4, 4)

        # 展平
        x = x.view(-1, 16 * 4 * 4)                      # (B, 16, 4, 4) → (B, 256)

        # 全连接层1
        x = self.fc1(x)                                 # (B, 256) → (B, 120)
        x = torch.relu(x)

        # 全连接层2
        x = self.fc2(x)                                 # (B, 120) → (B, 84)
        x = torch.relu(x)

        # 输出层
        x = self.fc3(x)                                 # (B, 84) → (B, 10)

        return x



"""
通用 CNN 模型

适用于图像分类任务（MNIST, CIFAR-10, Fashion-MNIST 等）
"""

import torch
import torch.nn as nn
import torch.nn.functional as F
import numpy as np
from typing import List, Optional, Tuple
from ...registry import model


@model(
    name='simple_cnn',
    description='简单的CNN模型 - 可配置的卷积神经网络，适用于图像分类',
    version='1.0',
    author='Federation Framework',
    model_type='cnn',
    framework='pytorch'
)
class SimpleCNN(nn.Module):
    """
    简单的 CNN 模型

    可配置的卷积神经网络，适用于多种图像分类任务
    """

    def __init__(
        self,
        input_shape: Tuple[int, int, int] = (28, 28, 1),
        num_classes: int = 10,
        conv_channels: List[int] = None,
        dense_units: int = 128,
        dropout_rate: float = 0.5,
    ):
        """
        Args:
            input_shape: 输入形状 (height, width, channels)
            num_classes: 类别数量
            conv_channels: 卷积层通道数列表
            dense_units: 全连接层单元数
            dropout_rate: Dropout 比率
        """
        super(SimpleCNN, self).__init__()

        if conv_channels is None:
            conv_channels = [32, 64]

        self.input_shape = input_shape
        self.num_classes = num_classes
        self.conv_channels = conv_channels
        self.dense_units = dense_units
        self.dropout_rate = dropout_rate

        # 输入通道数（根据input_shape调整）
        # input_shape格式: (H, W, C)，需要转换为 (C, H, W) 供PyTorch使用
        in_channels = input_shape[2]

        # 构建卷积层
        self.conv_layers = nn.ModuleList()
        self.pool = nn.MaxPool2d(2, 2)

        prev_channels = in_channels
        for channels in conv_channels:
            self.conv_layers.append(
                nn.Conv2d(prev_channels, channels, kernel_size=3, padding=1)
            )
            prev_channels = channels

        # 计算卷积后的特征图大小
        # 每个MaxPool将尺寸减半
        feature_h = input_shape[0]
        feature_w = input_shape[1]
        for _ in conv_channels:
            feature_h = feature_h // 2
            feature_w = feature_w // 2

        self.flatten_size = conv_channels[-1] * feature_h * feature_w

        # 全连接层
        self.fc1 = nn.Linear(self.flatten_size, dense_units)
        self.dropout = nn.Dropout(dropout_rate)
        self.fc2 = nn.Linear(dense_units, num_classes)

    def forward(self, x):
        """
        前向传播

        Args:
            x: 输入张量，形状 (N, C, H, W)

        Returns:
            输出张量，形状 (N, num_classes)
        """
        # 卷积层 + 池化
        for conv in self.conv_layers:
            x = self.pool(F.relu(conv(x)))

        # 展平
        x = x.reshape(-1, self.flatten_size)

        # 全连接层
        x = F.relu(self.fc1(x))
        x = self.dropout(x)
        x = self.fc2(x)

        return x

    def get_weights(self) -> List[np.ndarray]:
        """
        获取模型权重（转换为numpy数组）

        Returns:
            权重列表
        """
        return [param.data.cpu().numpy() for param in self.parameters()]

    def set_weights(self, weights: List[np.ndarray]):
        """
        设置模型权重（从numpy数组）

        Args:
            weights: 权重列表
        """
        with torch.no_grad():
            for param, weight in zip(self.parameters(), weights):
                param.data = torch.from_numpy(weight).to(param.device)

    def train_step(
        self,
        x_train: np.ndarray,
        y_train: np.ndarray,
        epochs: int = 1,
        batch_size: int = 32,
        learning_rate: float = 0.001,
        device: str = 'cpu',
        verbose: bool = False
    ) -> dict:
        """
        训练模型

        Args:
            x_train: 训练数据，形状 (N, H, W, C)
            y_train: 训练标签，形状 (N,)
            epochs: 训练轮数
            batch_size: 批次大小
            learning_rate: 学习率
            device: 设备 ('cpu' 或 'cuda')
            verbose: 是否打印训练信息

        Returns:
            训练历史字典
        """
        self.to(device)
        self.train()

        # 转换数据格式: (N, H, W, C) -> (N, C, H, W)
        x_train = torch.from_numpy(x_train).permute(0, 3, 1, 2).float().to(device)
        y_train = torch.from_numpy(y_train).long().to(device)

        # 创建数据加载器
        dataset = torch.utils.data.TensorDataset(x_train, y_train)
        dataloader = torch.utils.data.DataLoader(
            dataset, batch_size=batch_size, shuffle=True
        )

        # 优化器和损失函数
        optimizer = torch.optim.Adam(self.parameters(), lr=learning_rate)
        criterion = nn.CrossEntropyLoss()

        history = {'loss': [], 'accuracy': []}

        for epoch in range(epochs):
            epoch_loss = 0.0
            correct = 0
            total = 0

            for inputs, labels in dataloader:
                # 前向传播
                outputs = self(inputs)
                loss = criterion(outputs, labels)

                # 反向传播
                optimizer.zero_grad()
                loss.backward()
                optimizer.step()

                # 统计
                epoch_loss += loss.item() * inputs.size(0)
                _, predicted = torch.max(outputs, 1)
                total += labels.size(0)
                correct += (predicted == labels).sum().item()

            # 记录
            avg_loss = epoch_loss / total
            accuracy = correct / total
            history['loss'].append(avg_loss)
            history['accuracy'].append(accuracy)

            if verbose:
                print(f'Epoch {epoch+1}/{epochs} - Loss: {avg_loss:.4f} - Accuracy: {accuracy:.4f}')

        return history

    def evaluate_model(
        self,
        x_test: np.ndarray,
        y_test: np.ndarray,
        batch_size: int = 32,
        device: str = 'cpu',
        verbose: bool = False
    ) -> Tuple[float, float]:
        """
        评估模型

        Args:
            x_test: 测试数据，形状 (N, H, W, C)
            y_test: 测试标签，形状 (N,)
            batch_size: 批次大小
            device: 设备
            verbose: 是否打印信息

        Returns:
            (loss, accuracy)
        """
        self.to(device)
        self.eval()

        # 转换数据格式
        x_test = torch.from_numpy(x_test).permute(0, 3, 1, 2).float().to(device)
        y_test = torch.from_numpy(y_test).long().to(device)

        # 创建数据加载器
        dataset = torch.utils.data.TensorDataset(x_test, y_test)
        dataloader = torch.utils.data.DataLoader(
            dataset, batch_size=batch_size, shuffle=False
        )

        criterion = nn.CrossEntropyLoss()

        total_loss = 0.0
        correct = 0
        total = 0

        with torch.no_grad():
            for inputs, labels in dataloader:
                outputs = self(inputs)
                loss = criterion(outputs, labels)

                total_loss += loss.item() * inputs.size(0)
                _, predicted = torch.max(outputs, 1)
                total += labels.size(0)
                correct += (predicted == labels).sum().item()

        avg_loss = total_loss / total
        accuracy = correct / total

        if verbose:
            print(f'Test Loss: {avg_loss:.4f} - Test Accuracy: {accuracy:.4f}')

        return avg_loss, accuracy

    def save(self, path: str):
        """
        保存模型

        Args:
            path: 保存路径
        """
        torch.save({
            'state_dict': self.state_dict(),
            'config': {
                'input_shape': self.input_shape,
                'num_classes': self.num_classes,
                'conv_channels': self.conv_channels,
                'dense_units': self.dense_units,
                'dropout_rate': self.dropout_rate,
            }
        }, path)

    def load(self, path: str, device: str = 'cpu'):
        """
        加载模型

        Args:
            path: 模型路径
            device: 设备
        """
        checkpoint = torch.load(path, map_location=device)
        self.load_state_dict(checkpoint['state_dict'])
        return self

    @classmethod
    def from_pretrained(cls, path: str, device: str = 'cpu') -> 'SimpleCNN':
        """
        从保存的模型加载

        Args:
            path: 模型路径
            device: 设备

        Returns:
            加载的模型实例
        """
        checkpoint = torch.load(path, map_location=device)
        config = checkpoint['config']
        model = cls(**config)
        model.load_state_dict(checkpoint['state_dict'])
        return model



"""
内置模型

注意:根据框架设计原则,我们不再定义自定义模型基类
直接使用 PyTorch nn.Module 和 Transformers 模型

同时提供一些常用的模型实现
"""

from .cnn import SimpleCNN
from .mnist_cnn import MNISTCNNModel
from .mnist_lenet import MNISTLeNetModel
from .paper_cnn import PaperCNN, CIFAR10_PaperCNN, MNIST_PaperCNN
from .paper_mlp import PaperMLP, Adult_PaperMLP, FCUBE_PaperMLP
from .cifar10_cnn import CNNModel, SimpleCNNModel
from .resnet import ResNet18_CIFAR, ResNet34_CIFAR

__all__ = [
    "SimpleCNN",
    "MNISTCNNModel",
    "MNISTLeNetModel",
    "PaperCNN",
    "CIFAR10_PaperCNN",
    "MNIST_PaperCNN",
    "PaperMLP",
    "Adult_PaperMLP",
    "FCUBE_PaperMLP",
    "CNNModel",
    "SimpleCNNModel",
    "ResNet18_CIFAR",
    "ResNet34_CIFAR",
]



"""
配置类型定义

纯数据类，只定义配置的结构，不包含加载逻辑。
所有配置类都是不可变的 dataclass，便于序列化和测试。

架构设计：
    GlobalConfig          # 全局共享配置（exp_name, run_name 等）
    ├── LogConfig         # 日志配置
    ├── TrackerConfig     # 追踪配置
    ├── TransportConfig   # 传输层配置
    └── NodeConfig        # 节点配置（包含以上所有）
"""

from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Dict, List, Optional, Union
from enum import Enum


# ==================== 枚举类型 ====================

class LogLevel(str, Enum):
    """日志级别"""
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"
    CRITICAL = "CRITICAL"


class TransportMode(str, Enum):
    """传输模式"""
    MEMORY = "memory"
    GRPC = "grpc"


class BackoffStrategy(str, Enum):
    """退避策略"""
    CONSTANT = "constant"
    LINEAR = "linear"
    EXPONENTIAL = "exponential"


class NodeRole(str, Enum):
    """节点角色"""
    TRAINER = "trainer"
    LEARNER = "learner"
    BOTH = "both"


# ==================== 全局配置 ====================

@dataclass
class GlobalConfig:
    """
    全局共享配置
    
    这些字段会自动同步到子配置（LogConfig、TrackerConfig 等），
    避免在多处重复设置相同的值。
    
    Attributes:
        exp_name: 实验名称，用于组织日志和追踪数据
        run_name: 运行名称，None 则自动生成（基于时间戳）
        log_dir: 日志根目录
        
    Example:
        global_config = GlobalConfig(
            exp_name="fedavg_mnist",
            run_name="run_001",
            log_dir="./logs",
        )
    """
    exp_name: str = "default"
    run_name: Optional[str] = None
    log_dir: str = "./logs"
    
    def generate_run_name(self) -> str:
        """生成默认的 run_name（基于时间戳）"""
        if self.run_name:
            return self.run_name
        return datetime.now().strftime("%Y%m%d_%H%M%S")
    
    def get_run_name(self) -> str:
        """获取 run_name，如果为 None 则生成"""
        return self.run_name or self.generate_run_name()


# ==================== 日志配置 ====================

@dataclass
class LogConfig:
    """
    日志配置
    
    基于 loguru 的日志配置，支持控制台和文件输出。
    
    Attributes:
        level: 文件日志级别
        console: 是否输出到控制台
        console_level: 控制台日志级别
        rotation: 日志轮转大小（如 "10 MB"）
        retention: 日志保留时间（如 "30 days"）
        compression: 压缩格式（如 "zip"）
        format: 日志格式字符串
        diagnose: 是否显示详细诊断信息
        log_dir: 日志目录
        exp_name: 实验名称
        
    Example:
        log_config = LogConfig(
            level="DEBUG",
            console=True,
            console_level="INFO",
            log_dir="./logs",
            exp_name="my_experiment",
        )
    """
    # 基础配置
    level: str = "INFO"
    console: bool = True
    console_level: str = "INFO"
    
    # 文件输出配置
    rotation: str = "10 MB"
    retention: str = "30 days"
    compression: str = "zip"
    
    # 格式配置
    format: str = (
        "{time:YYYY-MM-DD HH:mm:ss.SSS} | "
        "{level: <8} | "
        "{extra[node_id]} | "
        "{name}:{function}:{line} - "
        "{message}"
    )
    
    # 调试配置
    diagnose: bool = False
    
    # ===== 路径和实验信息 =====
    # 这些字段可以直接初始化，也可以由 ConfigManager 同步设置
    log_dir: Optional[str] = field(default="./logs")
    exp_name: Optional[str] = field(default=None)
    run_name: Optional[str] = field(default=None)
    
    # ===== 向后兼容的私有字段别名 =====
    # 保留这些属性以兼容旧代码
    @property
    def _exp_name(self) -> Optional[str]:
        return self.exp_name
    
    @_exp_name.setter
    def _exp_name(self, value: Optional[str]):
        object.__setattr__(self, 'exp_name', value)
    
    @property
    def _log_dir(self) -> Optional[str]:
        return self.log_dir
    
    @_log_dir.setter
    def _log_dir(self, value: Optional[str]):
        object.__setattr__(self, 'log_dir', value)
    
    @property
    def _run_name(self) -> Optional[str]:
        return self.run_name
    
    @_run_name.setter
    def _run_name(self, value: Optional[str]):
        object.__setattr__(self, 'run_name', value)
    
    def get_log_path(self) -> str:
        """
        获取完整的日志路径
        
        Returns:
            格式：{log_dir}/{exp_name}/{run_name}/
        """
        parts = [self.log_dir]
        if self._exp_name:
            parts.append(self._exp_name)
        if self._run_name:
            parts.append(self._run_name)
        return "/".join(parts)
    
    def __post_init__(self):
        """初始化后处理"""
        # 标准化日志级别为大写
        self.level = self.level.upper()
        self.console_level = self.console_level.upper()


# ==================== 追踪配置 ====================

@dataclass
class TrackerBackendConfig:
    """
    单个 Tracker Backend 配置
    
    Attributes:
        type: backend 类型（file, mlflow, wandb, tensorboard）
        args: 配置参数字典
        
    Example:
        backend = TrackerBackendConfig(
            type="mlflow",
            args={"tracking_uri": "http://localhost:5000"},
        )
    """
    type: str
    args: Optional[Dict[str, Any]] = None
    
    def get_args(self) -> Dict[str, Any]:
        """获取参数字典"""
        return self.args or {}


@dataclass
class MLflowConfig:
    """MLflow 追踪配置"""
    tracking_uri: str = "http://localhost:5000"
    experiment_name: Optional[str] = None  # None 则使用 exp_name
    run_name: Optional[str] = None         # None 则使用全局 run_name
    
    # 同步字段
    _exp_name: Optional[str] = field(default=None, repr=False, compare=False)
    _run_name: Optional[str] = field(default=None, repr=False, compare=False)
    
    def get_experiment_name(self) -> str:
        """获取实验名称，优先使用显式设置的值"""
        return self.experiment_name or self._exp_name or "default"
    
    def get_run_name(self) -> Optional[str]:
        """获取运行名称，优先使用显式设置的值"""
        return self.run_name or self._run_name


@dataclass
class WandbConfig:
    """Weights & Biases 追踪配置"""
    project: str = "federated-learning"
    entity: Optional[str] = None
    name: Optional[str] = None             # None 则使用全局 run_name
    tags: List[str] = field(default_factory=list)
    
    # 同步字段
    _exp_name: Optional[str] = field(default=None, repr=False, compare=False)
    _run_name: Optional[str] = field(default=None, repr=False, compare=False)
    
    def get_name(self) -> Optional[str]:
        """获取运行名称"""
        return self.name or self._run_name


@dataclass
class TensorBoardConfig:
    """TensorBoard 追踪配置"""
    log_dir: Optional[str] = None          # None 则自动生成
    
    # 同步字段
    _log_dir: Optional[str] = field(default=None, repr=False, compare=False)
    _exp_name: Optional[str] = field(default=None, repr=False, compare=False)
    _run_name: Optional[str] = field(default=None, repr=False, compare=False)
    
    def get_log_dir(self) -> str:
        """获取日志目录"""
        if self.log_dir:
            return self.log_dir
        # 自动生成：{log_dir}/{exp_name}/{run_name}/tensorboard
        parts = [self._log_dir or "./logs"]
        if self._exp_name:
            parts.append(self._exp_name)
        if self._run_name:
            parts.append(self._run_name)
        parts.append("tensorboard")
        return "/".join(parts)


@dataclass
class TrackerConfig:
    """
    训练追踪配置
    
    Attributes:
        enabled: 是否启用追踪
        tracking_dir: 追踪目录（相对于日志目录）
        backends: backend 配置列表
        
    Example:
        tracker = TrackerConfig(
            enabled=True,
            backends=[
                TrackerBackendConfig(type="file"),
                TrackerBackendConfig(
                    type="mlflow",
                    args={"tracking_uri": "./mlruns"},
                ),
            ],
        )
    """
    enabled: bool = True
    tracking_dir: str = "tracking"
    backends: Optional[List[Union[Dict[str, Any], TrackerBackendConfig]]] = None
    
    # 同步字段
    _exp_name: Optional[str] = field(default=None, repr=False, compare=False)
    _run_name: Optional[str] = field(default=None, repr=False, compare=False)
    _log_dir: Optional[str] = field(default=None, repr=False, compare=False)
    
    @property
    def exp_name(self) -> Optional[str]:
        """实验名称（从 GlobalConfig 同步）"""
        return self._exp_name
    
    @property
    def run_name(self) -> Optional[str]:
        """运行名称（从 GlobalConfig 同步）"""
        return self._run_name
    
    @property
    def log_dir(self) -> Optional[str]:
        """日志目录（从 GlobalConfig 同步）"""
        return self._log_dir
    
    def get_tracking_path(self) -> str:
        """
        获取完整的追踪路径
        
        Returns:
            格式：{log_dir}/{exp_name}/{run_name}/{tracking_dir}/
        """
        parts = [self._log_dir or "./logs"]
        if self._exp_name:
            parts.append(self._exp_name)
        if self._run_name:
            parts.append(self._run_name)
        parts.append(self.tracking_dir)
        return "/".join(parts)
    
    def get_backends(self) -> List[TrackerBackendConfig]:
        """获取标准化的 backend 列表"""
        if not self.backends:
            return []
        
        result = []
        for backend in self.backends:
            if isinstance(backend, TrackerBackendConfig):
                result.append(backend)
            elif isinstance(backend, dict):
                result.append(TrackerBackendConfig(
                    type=backend.get("type", "file"),
                    args=backend.get("args"),
                ))
        return result


# ==================== 传输配置 ====================

@dataclass
class GrpcConfig:
    """gRPC 配置"""
    host: str = "0.0.0.0"
    port: int = 50051
    max_message_size: int = 100 * 1024 * 1024  # 100MB
    
    def get_address(self) -> str:
        """获取完整地址"""
        return f"{self.host}:{self.port}"


@dataclass
class TransportConfig:
    """
    传输层配置
    
    Attributes:
        mode: 传输模式（memory 或 grpc）
        grpc: gRPC 配置
        
    Example:
        transport = TransportConfig(
            mode="grpc",
            grpc=GrpcConfig(host="0.0.0.0", port=50051),
        )
    """
    mode: str = "memory"
    grpc: GrpcConfig = field(default_factory=GrpcConfig)
    
    def __post_init__(self):
        """初始化后处理"""
        # 如果 grpc 是字典，转换为 GrpcConfig
        if isinstance(self.grpc, dict):
            self.grpc = GrpcConfig(**self.grpc)


@dataclass
class ConnectionRetryConfig:
    """
    连接重试配置
    
    Attributes:
        enabled: 是否启用重试
        max_retries: 最大重试次数（-1 表示无限重试）
        retry_interval: 重试间隔（秒）
        timeout: 总超时时间（秒）
        backoff: 退避策略
        backoff_factor: 退避因子
    """
    enabled: bool = True
    max_retries: int = 10
    retry_interval: float = 2.0
    timeout: float = 60.0
    backoff: str = "exponential"
    backoff_factor: float = 1.5


# ==================== 组件配置 ====================

@dataclass
class ComponentConfig:
    """
    通用组件配置
    
    用于 trainer, learner, aggregator, model 等组件。
    
    Attributes:
        type: 组件类型（如 "federated.trainer.fedavg"）
        args: 组件参数
        
    Example:
        trainer = ComponentConfig(
            type="federated.trainer.fedavg",
            args={"max_rounds": 100},
        )
    """
    type: str
    args: Optional[Dict[str, Any]] = None
    
    def get_args(self) -> Dict[str, Any]:
        """获取参数字典"""
        return self.args or {}


@dataclass
class DatasetConfig:
    """数据集配置"""
    type: str
    split: str = "train"
    args: Optional[Dict[str, Any]] = None
    partition: Optional[Dict[str, Any]] = None
    
    def __post_init__(self):
        # 自动将 split 注入 args
        if self.args is None:
            self.args = {}
        if "split" not in self.args:
            self.args["split"] = self.split
    
    def get_args(self) -> Dict[str, Any]:
        return self.args.copy() if self.args else {}


# ==================== 回调配置 ====================

@dataclass
class CallbackConfig:
    """
    回调配置
    
    Attributes:
        type: 回调类型
        args: 回调参数
        
    Example:
        callback = CallbackConfig(
            type="callbacks.early_stopping",
            args={"patience": 5},
        )
    """
    type: str
    args: Optional[Dict[str, Any]] = None
    
    def get_args(self) -> Dict[str, Any]:
        """获取参数字典"""
        return self.args or {}


# ==================== 通信节点配置 ====================

@dataclass
class NodeCommConfig:
    """
    通信节点配置（给 Node 使用）
    
    这是 NodeConfig 的子集，只包含通信相关的配置。
    Node 类只需要这些配置来初始化 node_comm。
    
    Attributes:
        node_id: 节点唯一标识符
        default_timeout: 默认超时时间（秒）
        debug: 是否开启调试模式
        advertised_address: 广播地址
        listen: 监听配置
        transport: 传输层配置
        serialization: 序列化配置
        heartbeat: 心跳配置
        
    Example:
        comm_config = NodeCommConfig(
            node_id="trainer",
            listen={"port": 50051},
            transport=TransportConfig(mode="grpc"),
        )
        node = Node(comm_config)
    """
    node_id: str
    default_timeout: float = 30.0
    debug: bool = False
    advertised_address: Optional[str] = None
    listen: Optional[Dict[str, Any]] = None
    transport: TransportConfig = field(default_factory=TransportConfig)
    serialization: Optional[Dict[str, Any]] = None
    heartbeat: Optional[Dict[str, Any]] = None
    
    @property
    def transport_mode(self) -> str:
        """获取传输模式"""
        return self.transport.mode
    
    @property
    def listen_port(self) -> Optional[int]:
        """获取监听端口"""
        if self.listen:
            return self.listen.get("port")
        return None
    
    @property
    def listen_host(self) -> str:
        """获取监听地址"""
        if self.listen:
            return self.listen.get("host", "0.0.0.0")
        return "0.0.0.0"
    
    @property
    def grpc_address(self) -> str:
        """获取 gRPC 地址"""
        return f"{self.transport.grpc.host}:{self.transport.grpc.port}"


# ==================== 节点配置 ====================

@dataclass
class NodeConfig:
    """
    节点配置
    
    联邦学习系统中单个节点的完整配置。
    
    Attributes:
        node_id: 节点唯一标识符
        role: 节点角色（trainer, learner, both）
        
        # 全局配置
        global_config: 全局共享配置
        
        # 连接配置
        listen: 监听配置（Trainer 使用）
        connect_to: 连接目标列表（Learner 使用）
        
        # 基础设施配置
        transport: 传输层配置
        connection_retry: 连接重试配置
        logging: 日志配置
        tracker: 追踪配置
        
        # 组件配置
        trainer: Trainer 组件配置
        learner: Learner 组件配置
        aggregator: Aggregator 组件配置
        model: 模型配置
        
        # 数据集配置
        datasets: 训练数据集列表
        test_datasets: 测试数据集列表
        dataset: 单个数据集（向后兼容）
        
        # 回调配置
        callbacks: 回调列表
        
        # 其他配置
        serialization: 序列化配置
        min_peers: Trainer 等待的最少对等节点数
        default_timeout: 默认超时时间
        heartbeat: 心跳配置
        
    Example:
        config = NodeConfig(
            node_id="trainer",
            role="trainer",
            global_config=GlobalConfig(exp_name="my_exp"),
            listen={"port": 50051},
            trainer=ComponentConfig(type="federated.trainer.fedavg"),
            aggregator=ComponentConfig(type="federated.aggregator.fedavg"),
        )
    """
    
    # ========== 基本信息 ==========
    node_id: str = ""
    role: str = "learner"
    extend: Optional[str] = None  # 继承的基础配置文件路径
    
    # ========== 全局配置 ==========
    global_config: Optional[GlobalConfig] = None
    
    # ========== 连接配置 ==========
    listen: Optional[Dict[str, Any]] = None
    connect_to: Optional[List[str]] = None
    
    # ========== 基础设施配置 ==========
    transport: TransportConfig = field(default_factory=TransportConfig)
    connection_retry: ConnectionRetryConfig = field(default_factory=ConnectionRetryConfig)
    logging: Optional[LogConfig] = None
    tracker: Optional[TrackerConfig] = None
    
    # ========== 组件配置 ==========
    trainer: Optional[Union[ComponentConfig, Dict[str, Any]]] = None
    learner: Optional[Union[ComponentConfig, Dict[str, Any]]] = None
    aggregator: Optional[Union[ComponentConfig, Dict[str, Any]]] = None
    model: Optional[Union[ComponentConfig, Dict[str, Any]]] = None
    
    # ========== 数据集配置 ==========
    datasets: Optional[List[Union[DatasetConfig, Dict[str, Any]]]] = None

    
    # ========== 回调配置 ==========
    callbacks: Optional[List[Union[CallbackConfig, Dict[str, Any]]]] = None
    
    # ========== 序列化配置 ==========
    serialization: Optional[Dict[str, Any]] = None
    
    # ========== 其他配置 ==========
    min_peers: int = 0
    default_timeout: float = 30.0
    heartbeat: Optional[Dict[str, Any]] = None
    
    # ========== 便捷属性 ==========
    
    @property
    def exp_name(self) -> str:
        """实验名称"""
        if self.global_config:
            return self.global_config.exp_name
        return "default"
    
    @property
    def run_name(self) -> Optional[str]:
        """运行名称"""
        if self.global_config:
            return self.global_config.run_name
        return None
    
    @property
    def log_dir(self) -> str:
        """日志目录"""
        if self.global_config:
            return self.global_config.log_dir
        return "./logs"
    
    def is_trainer(self) -> bool:
        """是否为 Trainer 角色"""
        return self.role in ("trainer", "both") or self.trainer is not None
    
    def is_learner(self) -> bool:
        """是否为 Learner 角色"""
        return self.role in ("learner", "both") or self.learner is not None
    
    def get_trainer_config(self) -> Optional[ComponentConfig]:
        """获取标准化的 Trainer 配置"""
        if self.trainer is None:
            return None
        if isinstance(self.trainer, ComponentConfig):
            return self.trainer
        return ComponentConfig(
            type=self.trainer.get("type", ""),
            args=self.trainer.get("args"),
        )
    
    def get_learner_config(self) -> Optional[ComponentConfig]:
        """获取标准化的 Learner 配置"""
        if self.learner is None:
            return None
        if isinstance(self.learner, ComponentConfig):
            return self.learner
        return ComponentConfig(
            type=self.learner.get("type", ""),
            args=self.learner.get("args"),
        )
    
    def get_aggregator_config(self) -> Optional[ComponentConfig]:
        """获取标准化的 Aggregator 配置"""
        if self.aggregator is None:
            return None
        if isinstance(self.aggregator, ComponentConfig):
            return self.aggregator
        return ComponentConfig(
            type=self.aggregator.get("type", ""),
            args=self.aggregator.get("args"),
        )
    
    def get_model_config(self) -> Optional[ComponentConfig]:
        """获取标准化的 Model 配置"""
        if self.model is None:
            return None
        if isinstance(self.model, ComponentConfig):
            return self.model
        return ComponentConfig(
            type=self.model.get("type", ""),
            args=self.model.get("args"),
        )
    
    def get_datasets(self, split: Optional[str] = None) -> List[DatasetConfig]:
        """
        获取数据集配置
        
        Args:
            split: 过滤条件
                - None: 返回全部
                - "train" / "test" / "valid": 返回对应类型
        """
        if not self.datasets:
            return []
        
        result = []
        for ds in self.datasets:
            if isinstance(ds, DatasetConfig):
                ds_config = ds
            elif isinstance(ds, dict):
                ds_config = DatasetConfig(
                    type=ds["type"],
                    split=ds.get("split", "train"),
                    args=ds.get("args"),
                    partition=ds.get("partition"),
                )
            else:
                continue
            
            if split is None or ds_config.split == split:
                result.append(ds_config)
        
        return result
    
    def get_train_datasets(self) -> List[DatasetConfig]:
        return self.get_datasets("train")
    
    def get_test_datasets(self) -> List[DatasetConfig]:
        return self.get_datasets("test")
    
    def get_valid_datasets(self) -> List[DatasetConfig]:
        return self.get_datasets("valid")
    
    def get_callbacks(self) -> List[CallbackConfig]:
        """获取标准化的回调列表"""
        if not self.callbacks:
            return []
        
        result = []
        for cb in self.callbacks:
            if isinstance(cb, CallbackConfig):
                result.append(cb)
            elif isinstance(cb, dict):
                result.append(CallbackConfig(
                    type=cb.get("type", ""),
                    args=cb.get("args"),
                ))
        return result
    
    def get_comm_config(self) -> "NodeCommConfig":
        """
        获取通信节点配置
        
        提取 Node 通信层需要的配置，创建 NodeCommConfig 实例。
        
        Returns:
            NodeCommConfig 实例
            
        Example:
            config = load_config("trainer.yaml")
            comm_config = config.get_comm_config()
            node = Node(comm_config)
        """
        return NodeCommConfig(
            node_id=self.node_id,
            default_timeout=self.default_timeout,
            debug=False,
            advertised_address=None,
            listen=self.listen,
            transport=self.transport,
            serialization=self.serialization,
            heartbeat=self.heartbeat,
        )


# ==================== 导出 ====================

__all__ = [
    # 枚举
    "LogLevel",
    "TransportMode",
    "BackoffStrategy",
    "NodeRole",
    
    # 全局配置
    "GlobalConfig",
    
    # 日志配置
    "LogConfig",
    
    # 追踪配置
    "TrackerBackendConfig",
    "MLflowConfig",
    "WandbConfig",
    "TensorBoardConfig",
    "TrackerConfig",
    
    # 传输配置
    "GrpcConfig",
    "TransportConfig",
    "ConnectionRetryConfig",
    
    # 组件配置
    "ComponentConfig",
    "DatasetConfig",
    "CallbackConfig",
    
    # 通信节点配置
    "NodeCommConfig",
    
    # 节点配置
    "NodeConfig",
]


"""
FedDBE (Federated Data-Free Knowledge Distillation based Ensemble) 学习器实现

从 methods/learners/fl/feddbe.py 迁移到新架构

FedDBE使用无数据知识蒸馏和模型集成来提高联邦学习性能

核心思想：
- 在服务器端集成多个客户端模型
- 使用生成的合成数据进行知识蒸馏
- 不需要访问原始训练数据
"""
from typing import Dict, Any, Optional, List
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.utils.data import DataLoader
import copy

from ....core.learner import Learner
from ....core.types import StepMetrics, EpochMetrics, EvalResult
from ....registry import learner


@learner('fl.feddbe', description='FedDBE: Federated Data-Free Knowledge Distillation based Ensemble')
class FedDBELearner(Learner):
    """FedDBE学习器 - 无数据知识蒸馏集成

    配置示例:
    {
        "learner": {
            "name": "fl.feddbe",
            "learning_rate": 0.01,
            "batch_size": 128,
            "local_epochs": 5,
            "optimizer": "SGD",
            "momentum": 0.9,
            "ensemble_distill": false,  # 是否使用集成蒸馏（简化版本不使用）
            "temperature": 3.0  # 蒸馏温度
        }
    }
    """

    def __init__(
        self,
        model,
        datasets,
        tracker=None,
        callbacks=None,
        config=None,
        node_id=None
    ):
        super().__init__(model, None, tracker, callbacks, config, node_id)

        self._datasets = datasets or {}

        # 从配置提取参数
        self.learning_rate = self._config.get('learning_rate', 0.01)
        self.batch_size = self._config.get('batch_size', 32)
        self.local_epochs = self._config.get('local_epochs', 5)
        self.momentum = self._config.get('momentum', 0.9)
        self.optimizer_type = self._config.get('optimizer', 'SGD').upper()

        # FedDBE特有参数
        self.ensemble_distill = self._config.get('ensemble_distill', False)
        self.temperature = self._config.get('temperature', 3.0)

        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

        # 组件占位符
        self.torch_model = None
        self._optimizer = None
        self._criterion = None
        self._train_loader = None
        self._test_loader = None

        # 存储历史模型用于集成（简化版本）
        self.historical_models = []

        self.logger.info(
            f"FedDBELearner {node_id} 初始化完成 "
            f"(lr={self.learning_rate}, ensemble_distill={self.ensemble_distill})"
        )

    async def setup(self, config: Dict) -> None:
        """初始化训练环境"""
        # 获取PyTorch模型
        self.torch_model = self._model.get_model()
        self.torch_model.to(self.device)

        # 创建优化器
        if self.optimizer_type == 'SGD':
            self._optimizer = optim.SGD(
                self.torch_model.parameters(),
                lr=self.learning_rate,
                momentum=self.momentum
            )
        elif self.optimizer_type == 'ADAM':
            self._optimizer = optim.Adam(
                self.torch_model.parameters(),
                lr=self.learning_rate
            )
        else:
            raise ValueError(f"不支持的优化器类型: {self.optimizer_type}")

        # 创建损失函数
        loss_name = self._config.get('loss', 'CrossEntropyLoss')
        if loss_name == 'CrossEntropyLoss':
            self._criterion = nn.CrossEntropyLoss()
        elif loss_name == 'MSELoss':
            self._criterion = nn.MSELoss()
        else:
            raise ValueError(f"不支持的损失函数: {loss_name}")

        # 创建数据加载器
        train_datasets = self._datasets.get("train", [])
        if train_datasets:
            self._train_loader = DataLoader(
                train_datasets[0],
                batch_size=self.batch_size,
                shuffle=True
            )

        test_datasets = self._datasets.get("test", [])
        if test_datasets:
            self._test_loader = DataLoader(
                test_datasets[0],
                batch_size=self.batch_size,
                shuffle=False
            )

        self.logger.info(f"Setup完成: train_samples={len(train_datasets[0]) if train_datasets else 0}")

    def ensemble_prediction(self, data):
        """
        集成多个模型的预测

        Args:
            data: 输入数据

        Returns:
            集成后的预测
        """
        # 当前模型预测
        output = self.torch_model(data)

        # 如果有历史模型，进行集成
        if self.ensemble_distill and len(self.historical_models) > 0:
            ensemble_outputs = [output]

            with torch.no_grad():
                for hist_model in self.historical_models:
                    hist_model.eval()
                    hist_output = hist_model(data)
                    ensemble_outputs.append(hist_output)

            # 平均集成
            output = torch.stack(ensemble_outputs).mean(dim=0)

        return output

    async def train_epoch(self, epoch: int) -> EpochMetrics:
        """单轮训练 - FedDBE训练循环"""
        self.torch_model.train()

        total_loss = 0.0
        total_ce_loss = 0.0
        total_distill_loss = 0.0
        total_correct = 0
        total_samples = 0

        for data, target in self._train_loader:
            data, target = data.to(self.device), target.to(self.device)

            self._optimizer.zero_grad()

            # 前向传播
            output = self.torch_model(data)
            ce_loss = self._criterion(output, target)

            # 如果启用集成蒸馏，添加蒸馏损失
            distill_loss = torch.tensor(0.0, device=self.device)
            if self.ensemble_distill and len(self.historical_models) > 0:
                ensemble_output = self.ensemble_prediction(data)

                # KL散度蒸馏损失
                student_probs = F.log_softmax(output / self.temperature, dim=1)
                teacher_probs = F.softmax(ensemble_output / self.temperature, dim=1)

                distill_loss = F.kl_div(
                    student_probs,
                    teacher_probs.detach(),
                    reduction='batchmean'
                ) * (self.temperature ** 2)

                # 组合损失
                loss = 0.5 * ce_loss + 0.5 * distill_loss
            else:
                loss = ce_loss

            loss.backward()
            self._optimizer.step()

            total_loss += loss.item() * data.size(0)
            total_ce_loss += ce_loss.item() * data.size(0)
            total_distill_loss += distill_loss.item() * data.size(0)

            pred = output.argmax(dim=1, keepdim=True)
            total_correct += pred.eq(target.view_as(pred)).sum().item()
            total_samples += data.size(0)

        avg_loss = total_loss / total_samples
        avg_accuracy = total_correct / total_samples

        self.logger.info(
            f"[{self._node_id}] Epoch {epoch}: "
            f"Loss={avg_loss:.4f} (CE={total_ce_loss/total_samples:.4f}, "
            f"Distill={total_distill_loss/total_samples:.4f}), Acc={avg_accuracy:.4f}"
        )

        epoch_metrics = EpochMetrics(
            epoch=epoch,
            avg_loss=avg_loss,
            total_samples=total_samples,
            metrics={
                'accuracy': avg_accuracy,
                'ce_loss': total_ce_loss / total_samples,
                'distill_loss': total_distill_loss / total_samples
            }
        )

        # 增加全局 epoch 计数器（用于 MLflow step）
        self._global_epoch_counter += 1

        # 触发 epoch 结束回调
        if self._callbacks:
            await self._callbacks.on_epoch_end(self, epoch, epoch_metrics)

        return epoch_metrics

    async def fit(self, config: Optional[Dict] = None):
        """训练模型 - 保存历史模型"""
        # 调用父类的fit方法
        await super().fit(config)

        # 保存当前模型到历史（用于下一轮集成）
        if self.ensemble_distill:
            self.historical_models.append(copy.deepcopy(self.torch_model))
            # 限制历史模型数量
            if len(self.historical_models) > 3:
                self.historical_models.pop(0)

    async def evaluate_model(self, config: Optional[Dict] = None) -> EvalResult:
        """评估模型 - 使用集成预测"""
        self.torch_model.eval()

        loader = self._test_loader if self._test_loader else self._train_loader

        total_loss = 0.0
        total_correct = 0
        total_samples = 0

        with torch.no_grad():
            for data, target in loader:
                data, target = data.to(self.device), target.to(self.device)

                # 使用集成预测
                output = self.ensemble_prediction(data)

                loss = self._criterion(output, target)
                total_loss += loss.item() * data.size(0)

                pred = output.argmax(dim=1, keepdim=True)
                total_correct += pred.eq(target.view_as(pred)).sum().item()
                total_samples += data.size(0)

        return EvalResult(
            num_samples=total_samples,
            metrics={
                'loss': total_loss / total_samples,
                'accuracy': total_correct / total_samples,
                'num_historical_models': len(self.historical_models)
            }
        )

    async def get_weights(self) -> Dict[str, Any]:
        """获取模型权重"""
        return {name: param.data.clone() for name, param in self.torch_model.state_dict().items()}

    async def set_weights(self, weights: Dict[str, Any]):
        """设置模型权重"""
        # 转换为torch tensor
        torch_weights = {}
        for k, v in weights.items():
            if torch.is_tensor(v):
                torch_weights[k] = v
            else:
                torch_weights[k] = torch.from_numpy(v)

        self.torch_model.load_state_dict(torch_weights)

        self.logger.debug(f"[{self._node_id}] FedDBE: Updated model")



"""
Tracker 同步 Callback

确保所有节点记录到同一个 run/session
"""

from typing import Dict, Any, Optional
from .sync_callback import SyncCallback
from ..registry import register
from ..infra import get_module_logger

logger = get_module_logger(__name__)


@register("federated.callback.tracker_sync")
class TrackerSyncCallback(SyncCallback):
    """
    Tracker 同步 Callback

    功能：
    - Trainer 端：收集 Tracker 的 run_id/session_id
    - Learner 端：加入到相同的 run/session

    这样可以确保所有节点的记录都在同一个运行会话中

    Example:
        # Trainer 端配置
        config = {
            "tracker": {"type": "mlflow", "args": {...}},
            "callbacks": [
                {"type": "tracker_sync", "args": {}}
            ]
        }

        # Learner 端配置
        config = {
            "tracker": {"type": "mlflow", "args": {...}},
            # Learner 端也需要配置 tracker_sync
        }

        # 训练开始前会自动同步
    """

    def __init__(self, tracker=None):
        """
        Args:
            tracker: Tracker 实例（通常由框架自动注入）
        """
        self.tracker = tracker

    async def collect_sync_info(self) -> Dict[str, Any]:
        """
        收集 Tracker 同步信息（Trainer 端）

        遍历所有 Tracker 后端，收集 run_id/session_id
        """
        if self.tracker is None:
            return {}

        sync_info = {}

        # 如果是 CompositeTracker，遍历所有后端
        if hasattr(self.tracker, "trackers"):
            for tracker in self.tracker.trackers:
                if hasattr(tracker, "get_sync_info"):
                    info = tracker.get_sync_info()
                    sync_info.update(info)

        # 单个 Tracker
        elif hasattr(self.tracker, "get_sync_info"):
            sync_info = self.tracker.get_sync_info()

        logger.info(f"Collected tracker sync info: {list(sync_info.keys())}")
        return sync_info

    async def apply_sync_info(self, sync_info: Dict[str, Any]):
        """
        应用 Tracker 同步信息（Learner 端）

        加入到对应的 run/session
        """
        if self.tracker is None or not sync_info:
            return

        # 如果是 CompositeTracker
        if hasattr(self.tracker, "trackers"):
            for tracker in self.tracker.trackers:
                self._apply_to_single_tracker(tracker, sync_info)

        # 单个 Tracker
        else:
            self._apply_to_single_tracker(self.tracker, sync_info)

        logger.info("Tracker synchronized")

    def _apply_to_single_tracker(self, tracker, sync_info: Dict[str, Any]):
        """
        应用到单个 Tracker

        Args:
            tracker: Tracker 实例
            sync_info: 同步信息
        """
        try:
            # MLflow
            if hasattr(tracker, "join_run") and "mlflow_run_id" in sync_info:
                tracker.join_run(sync_info["mlflow_run_id"])
                logger.debug(f"Joined MLflow run: {sync_info['mlflow_run_id']}")

            # Wandb
            if hasattr(tracker, "join_session") and "wandb_session_id" in sync_info:
                tracker.join_session(sync_info["wandb_session_id"])
                logger.debug(f"Joined Wandb session: {sync_info['wandb_session_id']}")

            # Loguru 无需同步

        except Exception as e:
            logger.error(f"Failed to sync tracker {tracker.__class__.__name__}: {e}")



"""
内置训练器实现
"""

from typing import Any, Dict, List, Optional
import asyncio

from ...core.trainer import Trainer
from ...core.types import ClientUpdate, TrainResult, RoundResult, RoundMetrics
from ...registry import trainer
from ...infra.logging import get_module_logger

logger = get_module_logger(__name__)


@trainer(
    name='default',
    description='默认FedAvg训练器 - 标准联邦平均训练流程',
    version='1.0',
    author='Federation Framework',
    algorithms=['fedavg']
)
class DefaultTrainer(Trainer):
    """
    默认 FedAvg 训练器

    标准联邦平均训练流程：
    1. 广播初始权重
    2. 轮次循环：
       a. 选择客户端
       b. 收集更新
       c. 聚合
       d. 广播新权重
       e. 评估（可选）
    """

    async def train_round(self, round_num: int) -> RoundResult:
        """
        单轮次训练 - 处理一个联邦学习轮次

        Args:
            round_num: 轮次编号

        Returns:
            RoundResult: 包含更新、权重、指标
        """
        # 触发轮次开始回调
        if self.callbacks:
            await self.callbacks.on_round_begin(self, round_num, {})

        config = self.config
        client_fraction = config.get("client_fraction", 1.0)
        fit_config = config.get("fit_config", {"epochs": 5})
        eval_interval = config.get("eval_interval", 10)

        # DEBUG: 输出配置信息
        self.logger.info(f"[Round {round_num}] DEBUG: config keys = {list(config.keys())}")
        self.logger.info(f"[Round {round_num}] DEBUG: client_fraction = {client_fraction}")

        # 1. 选择学习器
        connected = self.get_connected_learners()
        self.logger.info(f"[Round {round_num}] DEBUG: len(connected) = {len(connected)}")
        
        num_selected = max(1, int(len(connected) * client_fraction))
        self.logger.info(f"[Round {round_num}] DEBUG: num_selected = {num_selected}")
        
        selected = self.select_learners(num_selected, strategy="random")

        # DEBUG: 记录所有已连接和选中的学习器
        connected_ids = [getattr(l, '_target_id', 'unknown') for l in connected]
        selected_ids = [getattr(l, '_target_id', 'unknown') for l in selected]
        self.logger.debug(f"[Round {round_num}] 已连接学习器: {connected_ids}")
        self.logger.debug(f"[Round {round_num}] 选中学习器: {selected_ids}")

        self.logger.info(f"轮次 {round_num}: 选择了 {len(selected)} 个学习器")

        # 2. 收集训练结果（把 fit_config 作为位置参数传递）
        self.logger.debug(f"[Round {round_num}] 开始向 {selected_ids} 发送 fit 请求，配置: {fit_config}")
        results = await self.collect_results(selected, "fit", fit_config)
        self.logger.debug(f"[Round {round_num}] 收集到 {len(results)} 个响应")

        # 3. 过滤成功的结果并创建更新
        updates = []
        for i, (learner, result) in enumerate(zip(selected, results)):
            learner_id = getattr(learner, '_target_id', f'learner_{i}')

            # CRITICAL DEBUG: 强制记录每个learner的返回结果
            self.logger.debug(f"[Round {round_num}] Learner {learner_id} 返回: type={type(result).__name__}, is_Exception={isinstance(result, Exception)}")

            # 详细调试信息
            # self.logger.debug(f"[DEBUG-{learner_id}] result.__class__: {result.__class__}")
            # self.logger.debug(f"[DEBUG-{learner_id}] result.__class__.__module__: {result.__class__.__module__}")
            # self.logger.debug(f"[DEBUG-{learner_id}] TrainResult class: {TrainResult}")
            # self.logger.debug(f"[DEBUG-{learner_id}] TrainResult.__module__: {TrainResult.__module__}")
            # self.logger.debug(f"[DEBUG-{learner_id}] isinstance check: {isinstance(result, TrainResult)}")
            # self.logger.debug(f"[DEBUG-{learner_id}] type match: {type(result) == TrainResult}")
            # self.logger.debug(f"[DEBUG-{learner_id}] hasattr num_samples: {hasattr(result, 'num_samples')}")
            if hasattr(result, 'num_samples'):
                self.logger.debug(f"[DEBUG-{learner_id}] result.num_samples: {result.num_samples}")

            if isinstance(result, TrainResult):
                self.logger.info(f"samples={result.num_samples}, metrics_type={type(result.metrics)}")

            if isinstance(result, Exception):
                # DEBUG: 记录失败的学习器
                self.logger.error(f"[Round {round_num}] Learner {learner_id} 失败: {type(result).__name__}: {result}")
                # 使用 train_logger 记录完整的异常堆栈
                self.logger.exception(f"学习器失败", exc_info=result)
                raise result

            # 兼容性检查: 支持从不同模块路径导入的 TrainResult
            # (src.core.types.TrainResult 和 federation.core.types.TrainResult)
            is_train_result = isinstance(result, TrainResult) or (
                type(result).__name__ == 'TrainResult' and
                hasattr(result, 'num_samples') and
                hasattr(result, 'metrics')
            )

            if is_train_result:
                # DEBUG: 记录成功的学习器
                # result.metrics 是 TrainMetrics 对象，有 final_loss 和 metrics 字典
                loss_value = result.metrics.final_loss if hasattr(result.metrics, 'final_loss') else result.metrics.metrics.get('loss', 'N/A')
                self.logger.debug(f"[Round {round_num}] Learner {learner_id} 成功: samples={result.num_samples}, loss={loss_value}")
                updates.append(ClientUpdate.from_result(learner_id, result))

        if not updates:
            self.logger.info(f"轮次 {round_num}: 没有成功的更新", level="error")
            raise RuntimeError(f"轮次 {round_num}: 所有学习器都失败了")

        # 4. 聚合
        self.logger.debug(f"[Round {round_num}] 开始聚合，updates数量: {len(updates)}")
        self.logger.debug(f"[Round {round_num}] updates来源: {[u.client_id for u in updates]}")
        new_weights = self.aggregator.aggregate(updates, self.model)
        if self.model:
            self.model.set_weights(new_weights)
        self.logger.info(f"轮次 {round_num}: 聚合完成")

        # 5. 广播新权重到选中的学习器（而非所有学习器）
        self.logger.info(f"[Round {round_num}] 开始广播新权重到 {len(selected)} 个选中的学习器")
        self.logger.debug(f"[Round {round_num}] 广播目标: {[getattr(l, '_target_id', 'unknown') for l in selected]}")
        await self.broadcast_to_selected(selected, "set_weights", new_weights)
        self.logger.info(f"[Round {round_num}] 广播完成")

        # 6. 聚合后立即评估全局模型（如果配置）
        post_agg_metrics = {}
        if config.get("evaluate_after_aggregation", False):
            post_agg_metrics = await self._evaluate_global_model(round_num)

        # 7. 计算轮次指标（训练指标，不记录到tracker）
        round_metrics = self._compute_round_metrics(updates, round_num)
        # 合并聚合后评估指标
        round_metrics.metrics.update(post_agg_metrics)

        # 简化日志：只显示数据统计，不显示训练准确率（因为没意义）
        self.logger.info(
            f"轮次 {round_num}: "
            f"客户端数量={round_metrics.num_clients}, "
            f"训练样本数={round_metrics.total_samples}"
        )

        # 8. 定期评估（在所有客户端上评估，如果到了评估间隔）
        if eval_interval > 0 and round_num % eval_interval == 0:
            eval_metrics = await self._evaluate_round(round_num, selected)
            # 将评估指标合并到轮次指标
            round_metrics.metrics.update(eval_metrics)

            # 显著的日志输出：评估结果
            if eval_metrics:
                eval_str = ", ".join([f"{k}={v:.4f}" for k, v in eval_metrics.items()])
                self.logger.info(f"轮次 {round_num} 评估结果: {eval_str}")

        result = RoundResult(
            round_num=round_num,
            updates=updates,
            aggregated_weights=new_weights,
            metrics=round_metrics
        )

        # 触发轮次结束回调
        print(f"[DefaultTrainer-DEBUG] After round {round_num}: self.callbacks={self.callbacks}, bool={bool(self.callbacks)}")
        logger.info(f"[DefaultTrainer] After round {round_num}: self.callbacks={self.callbacks}, bool={bool(self.callbacks)}")

        if self.callbacks:
            print(f"[DefaultTrainer-DEBUG] Calling callbacks.on_round_end for round {round_num}")
            logger.info(f"[DefaultTrainer] Calling callbacks.on_round_end for round {round_num}")
            await self.callbacks.on_round_end(self, round_num, {"metrics": round_metrics})
            print(f"[DefaultTrainer-DEBUG] callbacks.on_round_end completed for round {round_num}")
            logger.info(f"[DefaultTrainer] callbacks.on_round_end completed for round {round_num}")
        else:
            print(f"[DefaultTrainer-DEBUG] NO CALLBACKS! self.callbacks is None or empty")
            logger.warning(f"[DefaultTrainer] NO CALLBACKS! self.callbacks is None or empty")

        return result

    def _compute_round_metrics(self, updates: List[ClientUpdate], round_num: int) -> RoundMetrics:
        """计算轮次指标"""
        if not updates:
            return RoundMetrics(
                round_num=round_num,
                num_clients=0,
                total_samples=0,
                metrics={}
            )

        total_samples = sum(u.num_samples for u in updates)

        # 聚合客户端指标
        all_metrics: Dict[str, List[float]] = {}
        for update in updates:
            for key, value in update.metrics.items():
                if key not in all_metrics:
                    all_metrics[key] = []
                all_metrics[key].append(value)

        # 计算平均值
        aggregated_metrics = {}
        for key, values in all_metrics.items():
            if len(values) > 0:  # 防止除零
                aggregated_metrics[f"avg_{key}"] = sum(values) / len(values)
            else:
                self.logger.warning(f"No values for metric '{key}', skipping average calculation")
                aggregated_metrics[f"avg_{key}"] = 0.0

        return RoundMetrics(
            round_num=round_num,
            num_clients=len(updates),
            total_samples=total_samples,
            metrics=aggregated_metrics
        )
    
    async def _evaluate_round(
        self,
        round_num: int,
        learners: List[Any],
    ) -> Dict[str, float]:
        """执行评估（在客户端测试集上）"""
        self.logger.info(f"轮次 {round_num} 开始评估...")

        eval_config = self.config.get("eval_config", {})
        results = await self.collect_results(learners, "evaluate", eval_config)

        # 聚合评估结果
        total_samples = 0
        weighted_metrics: Dict[str, float] = {}

        for result in results:
            if isinstance(result, Exception):
                continue

            samples = result.num_samples
            total_samples += samples

            for key, value in result.metrics.items():
                if key not in weighted_metrics:
                    weighted_metrics[key] = 0
                weighted_metrics[key] += value * samples

        # 计算加权平均
        eval_metrics = {}
        if total_samples > 0:
            for key, value in weighted_metrics.items():
                eval_metrics[f"eval_{key}"] = value / total_samples
            eval_metrics["eval_samples"] = total_samples

        # 注意：不在这里记录到 tracker，由上层统一记录
        # 这样可以避免重复记录

        return eval_metrics

    async def _evaluate_global_model(self, round_num: int) -> Dict[str, float]:
        """
        评估全局模型（在服务端测试集上）

        如果 Trainer 有 dataset，则在服务端数据上评估
        否则返回空字典
        """
        if not self.dataset:
            return {}

        self.logger.info(f"轮次 {round_num}: 评估全局模型")

        try:
            # 这里需要 Trainer 也有 evaluate 方法，或者使用模型直接评估
            # 简化实现：如果有 model 和 dataset，直接调用 model 的评估
            if hasattr(self.model, 'evaluate') and self.dataset:
                # 假设 model.evaluate 返回指标字典
                metrics = self.model.evaluate(self.dataset)
                global_metrics = {f"global_{k}": v for k, v in metrics.items()}
                if self.tracker:
                    self.tracker.log_metrics(global_metrics, step=round_num)
                return global_metrics
        except Exception as e:
            self.logger.exception(f"全局模型评估失败: {e}")

        return {}


@trainer(
    name='async',
    description='异步训练器 - 不等待所有客户端完成，收到更新即聚合',
    version='1.0',
    author='Federation Framework',
    algorithms=['async_fedavg']
)
class AsyncTrainer(Trainer):
    """
    异步训练器
    
    不等待所有客户端完成，收到更新即聚合
    """
    
    async def run(self) -> Dict[str, Any]:
        """运行异步训练流程"""
        config = self.config
        max_updates = config.get("max_updates", 1000)
        staleness_threshold = config.get("staleness_threshold", 10)
        fit_config = config.get("fit_config", {"epochs": 1})

        # 获取学习器
        learners = self.get_connected_learners()
        self.logger.info(f"异步训练开始，学习器数量: {len(learners)}")

        # 广播初始权重
        if self.model:
            await self.broadcast_to_learners("set_weights", self.model.get_weights())

        update_count = 0
        client_versions: Dict[str, int] = {}  # 客户端的模型版本
        current_version = 0

        while update_count < max_updates:
            connected = self.get_connected_learners()
            if not connected:
                self.logger.warning("没有连接的学习器，等待中...")
                await asyncio.sleep(1)
                continue

            # 对每个客户端发起训练请求
            for learner in connected:
                learner_id = getattr(learner, '_target_id', 'unknown')
                if learner_id not in client_versions:
                    client_versions[learner_id] = current_version

                # 检查过时程度
                staleness = current_version - client_versions[learner_id]
                if staleness > staleness_threshold and self.model:
                    # 发送最新权重
                    await self.broadcast_to_learners("set_weights", self.model.get_weights())
                    client_versions[learner_id] = current_version

            # 收集一个更新
            results = await self.collect_results(connected[:1], "fit", fit_config)

            for learner, result in zip(connected[:1], results):
                if isinstance(result, Exception):
                    continue

                learner_id = getattr(learner, '_target_id', 'unknown')
                update = ClientUpdate.from_result(learner_id, result)

                # 异步聚合（简化：直接平均）
                staleness = current_version - client_versions[learner_id]
                weight = 1.0 / (1.0 + staleness * 0.1)  # 过时惩罚

                if self.model:
                    current_weights = self.model.get_weights()
                    new_weights = self._weighted_average(
                        current_weights, update.weights, 1 - weight, weight
                    )
                    self.model.set_weights(new_weights)

                current_version += 1
                client_versions[learner_id] = current_version
                update_count += 1

                if update_count % 100 == 0:
                    self.logger.info(f"已处理 {update_count} 个更新")

        self.logger.success("异步训练完成")

        return {
            "total_updates": update_count,
            "final_version": current_version,
            "client_versions": client_versions,
        }


"""
内置模型实现
"""

from typing import Any, Dict, List, Optional
import pickle

from ...core.model import Model
from ...registry import register
from ...infra import get_module_logger

logger = get_module_logger(__name__)


@register("model.torch.wrapper")
class TorchModelWrapper(Model):
    """
    PyTorch 模型包装器
    
    将 PyTorch nn.Module 包装为 Model 接口
    """
    
    def __init__(self, model: Any, device: str = "cpu"):
        """
        初始化
        
        Args:
            model: PyTorch nn.Module
            device: 设备
        """
        self._model = model
        self._device = device
        
        try:
            import torch
            self._model = self._model.to(device)
        except ImportError:
            pass
    
    def get_weights(self) -> List[Any]:
        """获取模型权重"""
        try:
            import torch
            return [p.detach().cpu().numpy() for p in self._model.parameters()]
        except ImportError:
            return []
    
    def set_weights(self, weights: List[Any]) -> None:
        """设置模型权重"""
        try:
            import torch
            import numpy as np
            
            for param, w in zip(self._model.parameters(), weights):
                if isinstance(w, np.ndarray):
                    param.data = torch.from_numpy(w).to(self._device)
                else:
                    param.data = torch.tensor(w).to(self._device)
        except ImportError:
            pass
    
    def serialize(self) -> bytes:
        """序列化模型"""
        return pickle.dumps(self.get_weights())
    
    @classmethod
    def deserialize(cls, data: bytes) -> "TorchModelWrapper":
        """反序列化模型"""
        weights = pickle.loads(data)
        # 注意：这需要一个空模型来设置权重
        # 实际使用时需要提供模型结构
        raise NotImplementedError("Need model structure to deserialize")
    
    def train_mode(self) -> None:
        """设置为训练模式"""
        self._model.train()
    
    def eval_mode(self) -> None:
        """设置为评估模式"""
        self._model.eval()
    
    def to_device(self, device: str) -> "TorchModelWrapper":
        """移动到设备"""
        try:
            import torch
            self._device = device
            self._model = self._model.to(device)
        except ImportError:
            pass
        return self
    
    def num_parameters(self) -> int:
        """获取参数数量"""
        try:
            return sum(p.numel() for p in self._model.parameters())
        except:
            return 0


@register("model.torch.mlp")
class MLPModel(Model):
    """
    多层感知机模型
    
    简单的全连接网络
    """
    
    def __init__(
        self,
        input_size: int = 784,
        hidden_sizes: List[int] = None,
        output_size: int = 10,
        activation: str = "relu",
        device: str = "cpu",
    ):
        """
        初始化
        
        Args:
            input_size: 输入维度
            hidden_sizes: 隐藏层大小列表
            output_size: 输出维度
            activation: 激活函数
            device: 设备
        """
        hidden_sizes = hidden_sizes or [128, 64]
        self._device = device
        
        try:
            import torch
            import torch.nn as nn
            
            layers = []
            prev_size = input_size
            
            for hidden_size in hidden_sizes:
                layers.append(nn.Linear(prev_size, hidden_size))
                if activation == "relu":
                    layers.append(nn.ReLU())
                elif activation == "tanh":
                    layers.append(nn.Tanh())
                prev_size = hidden_size
            
            layers.append(nn.Linear(prev_size, output_size))
            
            self._model = nn.Sequential(*layers).to(device)
            
        except ImportError:
            logger.warning("PyTorch not available, using dummy model")
            self._model = None
            self._weights = []
    
    def get_weights(self) -> List[Any]:
        """获取模型权重"""
        if self._model is None:
            return self._weights
        
        try:
            import torch
            return [p.detach().cpu().numpy() for p in self._model.parameters()]
        except:
            return []
    
    def set_weights(self, weights: List[Any]) -> None:
        """设置模型权重"""
        if self._model is None:
            self._weights = weights
            return
        
        try:
            import torch
            import numpy as np
            
            for param, w in zip(self._model.parameters(), weights):
                if isinstance(w, np.ndarray):
                    param.data = torch.from_numpy(w).to(self._device)
                else:
                    param.data = torch.tensor(w).to(self._device)
        except:
            pass
    
    def serialize(self) -> bytes:
        """序列化模型"""
        return pickle.dumps(self.get_weights())
    
    @classmethod
    def deserialize(cls, data: bytes) -> "MLPModel":
        """反序列化模型"""
        model = cls()
        weights = pickle.loads(data)
        model.set_weights(weights)
        return model
    
    def train_mode(self) -> None:
        """设置为训练模式"""
        if self._model is not None:
            self._model.train()
    
    def eval_mode(self) -> None:
        """设置为评估模式"""
        if self._model is not None:
            self._model.eval()


@register("model.torch.cnn")
class CNNModel(Model):
    """
    卷积神经网络模型
    
    简单的 CNN 用于图像分类
    """
    
    def __init__(
        self,
        in_channels: int = 1,
        num_classes: int = 10,
        device: str = "cpu",
    ):
        """
        初始化
        
        Args:
            in_channels: 输入通道数
            num_classes: 类别数
            device: 设备
        """
        self._device = device
        
        try:
            import torch
            import torch.nn as nn
            
            self._model = nn.Sequential(
                nn.Conv2d(in_channels, 32, 3, 1),
                nn.ReLU(),
                nn.Conv2d(32, 64, 3, 1),
                nn.ReLU(),
                nn.MaxPool2d(2),
                nn.Flatten(),
                nn.Linear(9216, 128),
                nn.ReLU(),
                nn.Linear(128, num_classes),
            ).to(device)
            
        except ImportError:
            logger.warning("PyTorch not available, using dummy model")
            self._model = None
            self._weights = []
    
    def get_weights(self) -> List[Any]:
        """获取模型权重"""
        if self._model is None:
            return self._weights
        
        try:
            import torch
            return [p.detach().cpu().numpy() for p in self._model.parameters()]
        except:
            return []
    
    def set_weights(self, weights: List[Any]) -> None:
        """设置模型权重"""
        if self._model is None:
            self._weights = weights
            return
        
        try:
            import torch
            import numpy as np
            
            for param, w in zip(self._model.parameters(), weights):
                if isinstance(w, np.ndarray):
                    param.data = torch.from_numpy(w).to(self._device)
                else:
                    param.data = torch.tensor(w).to(self._device)
        except:
            pass
    
    def serialize(self) -> bytes:
        """序列化模型"""
        return pickle.dumps(self.get_weights())
    
    @classmethod
    def deserialize(cls, data: bytes) -> "CNNModel":
        """反序列化模型"""
        model = cls()
        weights = pickle.loads(data)
        model.set_weights(weights)
        return model
    
    def train_mode(self) -> None:
        """设置为训练模式"""
        if self._model is not None:
            self._model.train()
    
    def eval_mode(self) -> None:
        """设置为评估模式"""
        if self._model is not None:
            self._model.eval()


@register("model.numpy.linear")
class NumpyLinearModel(Model):
    """
    NumPy 线性模型
    
    不依赖深度学习框架的简单线性模型
    """
    
    def __init__(
        self,
        input_size: int = 784,
        output_size: int = 10,
    ):
        """
        初始化
        
        Args:
            input_size: 输入维度
            output_size: 输出维度
        """
        try:
            import numpy as np
            
            # 初始化权重
            self._W = np.random.randn(input_size, output_size) * 0.01
            self._b = np.zeros(output_size)
            
        except ImportError:
            self._W = [[0.0] * output_size for _ in range(input_size)]
            self._b = [0.0] * output_size
    
    def get_weights(self) -> List[Any]:
        """获取模型权重"""
        return [self._W, self._b]
    
    def set_weights(self, weights: List[Any]) -> None:
        """设置模型权重"""
        if len(weights) >= 2:
            self._W = weights[0]
            self._b = weights[1]
    
    def serialize(self) -> bytes:
        """序列化模型"""
        return pickle.dumps(self.get_weights())
    
    @classmethod
    def deserialize(cls, data: bytes) -> "NumpyLinearModel":
        """反序列化模型"""
        model = cls()
        weights = pickle.loads(data)
        model.set_weights(weights)
        return model
    
    def forward(self, x: Any) -> Any:
        """前向传播"""
        try:
            import numpy as np
            return np.dot(x, self._W) + self._b
        except:
            return x
